@@ -1,6 +1,7 @@
 #include <stdio.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <stdlib.h>
 #include <string.h>
 #include <errno.h>
 
@@ -18,6 +19,11 @@
 #define HOSTS_SSHGUARD_PREFIX "###sshguard###\n"
 #define HOSTS_SSHGUARD_SUFFIX "###sshguard###\n"
 
+typedef struct {
+    char addr[ADDRLEN];
+    int addrkind;
+    int service;
+} addr_service_t;
 
 int hosts_updatelist();
 int hosts_clearsshguardblocks(void);
@@ -25,6 +31,13 @@ int hosts_clearsshguardblocks(void);
 list_t hosts_blockedaddrs;
 FILE *hosts_file;
 
+size_t addr_service_meter(const void *el) { return sizeof(addr_service_t); }
+int addr_service_comparator(const void *a, const void *b) {
+    addr_service_t *A = (addr_service_t *)a;
+    addr_service_t *B = (addr_service_t *)b;
+    return !((strcmp(A->addr, B->addr) == 0) && (A->addrkind == B->addrkind) && (A->service == B->service));
+}
+
 int fw_init() {
     char buf[HOSTS_MAXCMDLEN];
     char tempflname[30];
@@ -64,8 +77,8 @@ int fw_init() {
     unlink(tempflname);
 
     list_init(&hosts_blockedaddrs);
-    list_attributes_copy(&hosts_blockedaddrs, list_meter_string, 1);
-    list_attributes_comparator(&hosts_blockedaddrs, list_comparator_string);
+    list_attributes_copy(&hosts_blockedaddrs, addr_service_meter, 1);
+    list_attributes_comparator(&hosts_blockedaddrs, addr_service_comparator);
 
     return FWALL_OK;
 }
@@ -77,15 +90,20 @@ int fw_fin() {
 }
 
 int fw_block(char *addr, int addrkind, int service) {
-    list_append(&hosts_blockedaddrs, addr);
+    addr_service_t ads;
+
+    strcpy(ads.addr, addr);
+    ads.service = service;
+    ads.addrkind = addrkind;
+    list_append(&hosts_blockedaddrs, &ads);
+
     return hosts_updatelist();
 }
 
 int fw_release(char *addr, int addrkind, int services) {
     int pos;
 
-    if (addrkind != ADDRKIND_IPv4) return FWALL_UNSUPP;
-    if ((pos = list_find(&hosts_blockedaddrs, addr)) < 0) {
+    if ((pos = list_locate(&hosts_blockedaddrs, addr)) < 0) {
         return FWALL_ERR;
     }
 
@@ -104,7 +122,6 @@ int hosts_updatelist() {
     char tempflname[30];
     FILE *tmp, *deny;
 
-
     /* open hosts.allow file */
     deny = fopen(HOSTSFILE_PATH, "r+");
     if (deny == NULL) {
@@ -138,22 +155,20 @@ int hosts_updatelist() {
     }
 
     if (list_size(& hosts_blockedaddrs) > 0) {
-        /* future use  */
-        /*
-        switch (service) {
-            case SERVICES_SSH:
-            fprintf(tmp, "\nsshd :");
-            break;
-        }
-        */
         unsigned int cnt;
-        fprintf(tmp, "sshd :");
+        addr_service_t *curr;
 
+        fprintf(tmp, "ALL :");
         for (cnt = 0; cnt < (int)list_size(&hosts_blockedaddrs); cnt++) {
-            fprintf(tmp, " %s", (char *)list_get_at(&hosts_blockedaddrs, cnt));
+            curr = (addr_service_t *)list_get_at(&hosts_blockedaddrs, cnt);
+
+            /* block lines differ depending on IP Version */
+            if (curr->addrkind == ADDRKIND_IPv4)
+                fprintf(tmp, " %s", curr->addr);
+            else fprintf(tmp, " [%s]", curr->addr);
         }
         fprintf(tmp, " : DENY\n");
-    }
+    }    
     fprintf(tmp, HOSTS_SSHGUARD_SUFFIX);
 
     /* getting to the end of the original block */
