@@ -43,7 +43,7 @@ int ipfw_rule_comparator(const void *a, const void *b) {
 }
 
 int fw_init() {
-    srandom(time(NULL));
+    srand(time(NULL));
     list_init(&addrrulenumbers);
     list_attributes_copy(& addrrulenumbers, ipfw_rule_meter, 1);
     list_attributes_comparator(& addrrulenumbers, ipfw_rule_comparator);
@@ -62,12 +62,14 @@ int fw_block(char *addr, int addrkind, int service) {
     struct addr_ruleno_s addendum;
 
     /* choose a random number to assign to IPFW rule */
-    ruleno = (random() % (IPFW_RULERANGE_MAX - IPFW_RULERANGE_MIN)) + IPFW_RULERANGE_MIN;
+    ruleno = (rand() % (IPFW_RULERANGE_MAX - IPFW_RULERANGE_MIN)) + IPFW_RULERANGE_MIN;
     switch (addrkind) {
         case ADDRKIND_IPv4:
             /* use ipfw */
             sprintf(command, IPFW_PATH "/ipfw");
+            sprintf(args, "add %u drop ip from %s to me", ruleno, addr);
             break;
+
         case ADDRKIND_IPv6:
 #ifdef FWALL_HAS_IP6FW
             /* use ip6fw if found */
@@ -76,13 +78,13 @@ int fw_block(char *addr, int addrkind, int service) {
             /* use ipfw, assume it supports IPv6 rules as well */
 	    	sprintf(command, IPFW_PATH "/ipfw");
 #endif
+            sprintf(args, "add %u drop ipv6 from %s to any", ruleno, addr);
             break;
+
         default:
             return FWALL_UNSUPP;
     }
-    /* build command arguments */
-    sprintf(args, "add %u drop ip from %s to me", ruleno, addr);
-            
+
     /* run command */
     ret = ipfwmod_runcommand(command, args);
     if (ret != 0) {
@@ -110,7 +112,7 @@ int fw_release(char *addr, int addrkind, int service) {
     /* retrieve ID of rule blocking "addr" */
     strcpy(data.addr, addr);
     data.addrkind = addrkind;
-    if ((pos = list_find(& addrrulenumbers, &data)) < 0) {
+    if ((pos = list_locate(& addrrulenumbers, &data)) < 0) {
         sshguard_log(LOG_ERR, "could not get back rule ID for address %s", addr);
         return FWALL_ERR;
     }
@@ -155,7 +157,9 @@ int fw_release(char *addr, int addrkind, int service) {
 int fw_flush(void) {
     struct addr_ruleno_s *data;
     char command[MAXIPFWCMDLEN], args[MAXIPFWCMDLEN];
-    int ret;
+    int ret = 0;
+
+    if (list_empty(& addrrulenumbers)) return FWALL_OK;
 
     list_iterator_start(&addrrulenumbers);
     while (list_iterator_hasnext(&addrrulenumbers)) {
