@@ -1,3 +1,23 @@
+/*
+ * Copyright (c) 2007,2008 Mij <mij@bitchx.it>
+ *
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ */
+
+/*
+ * SSHGuard. See http://www.sshguard.net
+ */
+
 
 #include <stdio.h>
 #include <string.h>
@@ -44,7 +64,7 @@ int whitelist_conf_init(void) {
     }
 
     /* IPv6 address regex */
-    if (regcomp(&wl_ip6reg, "^((([a-fA-F0-9]{1,4}:){2}(:[a-fA-F0-9]{1,4}){1,5})|(:(:[a-fA-F0-9]{1,4}){1,6}|([a-fA-F0-9]{1,4}:)(:[a-fA-F0-9]{1,4}){1,6}|([a-fA-F0-9]{1,4}:){2}(:[a-fA-F0-9]{1,4}){1,5}|([a-fA-F0-9]{1,4}:){3}(:[a-fA-F0-9]{1,4}){1,4}|([a-fA-F0-9]{1,4}:){4}(:[a-fA-F0-9]{1,4}){1,3}|([a-fA-F0-9]{1,4}:){5}(:[a-fA-F0-9]{1,4}){1,2}|([a-fA-F0-9]{1,4}:){6}:[a-fA-F0-9]{1,4}))$", REG_EXTENDED) != 0) {
+    if (regcomp(&wl_ip6reg, "^(((([a-fA-F0-9]{1,4}):){7}([a-fA-F0-9]{1,4}))|((([a-fA-F0-9]{1,4}))?::((([a-fA-F0-9]{1,4}):){1,5}(([a-fA-F0-9]{1,4})))?)|(::[fF0]{0,4}:)((25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9]([0-9])?)(.(25[0-5]|2[0-4][0-9]|1[0-9]{2}|[1-9]([0-9])?|0)){3}))$", REG_EXTENDED) != 0) {
         return -1;
     }
 
@@ -116,8 +136,8 @@ int whitelist_add(char *str) {
         sshguard_log(LOG_DEBUG, "whitelist: add '%s' as plain IPv4.", str);
         return whitelist_add_ipv4(str);
     } else if (regexec(&wl_ip6reg, str, 0, NULL, 0) == 0) {            /* plain IPv6 address */
-        sshguard_log(LOG_WARNING, "whitelist: IPv6 whitelisting not yet supported, skipping...");
-        return -1;
+        sshguard_log(LOG_DEBUG, "whitelist: add '%s' as plain IPv6.", str);
+        return whitelist_add_ipv6(str);
     } else if (regexec(&wl_hostreg, str, 0, NULL, 0) == 0) {        /* hostname to be resolved */
         sshguard_log(LOG_DEBUG, "whitelist: add '%s' as host.", str);
         return whitelist_add_host(str);
@@ -129,6 +149,10 @@ int whitelist_add(char *str) {
         pos = strrchr(buf, '/');
         *pos = '\0';
         masklen = (unsigned int)strtol(pos+1, (char **)NULL, 10);
+        if (masklen == 0 && pos[1] != '0') {
+            sshguard_log(LOG_WARNING, "whitelist: mask specified as '/%s' makes no sense.", pos+1);
+            return -1;
+        }
 
         if (masklen == 0 && errno != EINVAL) {
             /* could not convert the mask to an integer value */
@@ -137,13 +161,16 @@ int whitelist_add(char *str) {
         }
         if (regexec(&wl_ip4reg, buf, 0, NULL, 0) == 0) {
             if (masklen > 32) {     /* sanity check for netmask */
-                sshguard_log(LOG_WARNING, "whitelist: could not parse line \"%s\" as plain IP nor IP block nor host name", str);
+                sshguard_log(LOG_WARNING, "whitelist: mask length '%u' makes no sense for IPv4.", masklen);
                 return -1;
             }
             return whitelist_add_block4(buf, masklen);
         } else if (regexec(&wl_ip6reg, buf, 0, NULL, 0) == 0) {
-            sshguard_log(LOG_WARNING, "whitelist: IPv6 whitelisting not yet supported, skipping...");
-            return -1;
+            if (masklen > 128) {     /* sanity check for netmask */
+                sshguard_log(LOG_WARNING, "whitelist: mask length '%u' makes no sense for IPv6.", masklen);
+                return -1;
+            }
+            return whitelist_add_block6(buf, masklen);
         }
     } else {
         /* line not recognized */
@@ -172,6 +199,7 @@ int whitelist_add_block4(char *address, int masklen) {
 }
 
 int whitelist_add_block6(char *address, int masklen) {
+    sshguard_log(LOG_WARNING, "whitelist: IPv6 block whitelisting not yet supported, skipping...");
     return -1;
 }
 
@@ -188,6 +216,21 @@ int whitelist_add_ipv4(char *ip) {
 }
 
 int whitelist_add_ipv6(char *ip) {
+    addrblock_t ab;
+    int i;
+
+    ab.addrkind = ADDRKIND_IPv6;
+
+    if (inet_pton(AF_INET6, ip, &ab.address.ip6.address.s6_addr) != 1) {
+        sshguard_log(LOG_ERR, "whitelist: add ipv6: Could not add %s.", ip);
+        return -1;
+    }
+
+    for (i = 0; i < sizeof(struct in6_addr); i++)
+        ab.address.ip6.mask.s6_addr[i] = 0xFF;
+
+    list_append(&whitelist, & ab);
+    sshguard_log(LOG_DEBUG, "whitelist: add plain ip %s.", ip);
     return 0;
 }
 
@@ -216,20 +259,27 @@ int whitelist_add_host(char *host) {
 
 int whitelist_match(char *addr, int addrkind) {
     uint32_t addrent;
-    int i;
+    struct in6_addr addrent6;
+    int i, j;
     addrblock_t *entry;
 
     switch (addrkind) {
         case ADDRKIND_IPv4:
+            if (inet_pton(AF_INET, addr, &addrent) != 1) {
+                sshguard_log(LOG_WARNING, "whitelist: could not interpret ip address '%s'.", addr);
+                return 0;
+            }
             break;
         case ADDRKIND_IPv6:
+            if (inet_pton(AF_INET6, addr, &addrent6.s6_addr) != 1) {
+                sshguard_log(LOG_WARNING, "whitelist: could not interpret ip address '%s'.", addr);
+                return 0;
+            }
+            break;
         default:       /* not recognized */
             return 0;
     }
-    if (inet_pton(AF_INET, addr, &addrent) != 1) {
-        sshguard_log(LOG_WARNING, "whitelist: could not interpret ip address '%s'.", addr);
-        return 0;
-    }
+
     /* compare with every entry in the list */
     for (i = 0; (unsigned int)i < list_size(&whitelist); i++) {
         entry = (addrblock_t *)list_get_at(&whitelist, i);
@@ -240,10 +290,15 @@ int whitelist_match(char *addr, int addrkind) {
                     return 1;
                 }
                 break;
+            case ADDRKIND_IPv6:
+                for (j = 0; j < sizeof(addrent6); j++) {
+                    if ((entry->address.ip6.address.s6_addr[j] & entry->address.ip6.mask.s6_addr[j]) != (addrent6.s6_addr[j] & entry->address.ip6.mask.s6_addr[j]))
+                        return 0; 
+                }
+                return 1;
             default:
                 return 0;
         }
     }
     return 0;
 }
-
